/*
Copyright (c) 2009-2013, Stephane Sudre
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

- Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
- Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
- Neither the name of the WhiteBox nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import "EULA_PluginPane.h"

@implementation EULA_PluginPane

+ (NSDictionary *) defaultStorageOptionsWithPath:(NSString *) inPath
{
    NSDictionary * tOptionsDictionary;
    NSDictionary * tFontDictionary;
    NSString * tBaseURL=nil;
	
    tFontDictionary=[NSDictionary dictionaryWithObjectsAndKeys:[NSFont fontWithName:@"Helvetica" size:12.0f],NSFontAttributeName,nil];
    
	tBaseURL=[NSURL fileURLWithPath:inPath];
	
	tOptionsDictionary=[NSDictionary dictionaryWithObjectsAndKeys:tFontDictionary,@"DefaultAttributes",
																  tBaseURL,NSBaseURLDocumentOption,
                                                                  nil];
    
    return tOptionsDictionary;
}

- (void) awakeFromNib
{
	bundle_=[[[self section] bundle] retain];
	
	cachedLocalizedStrings_=[[NSMutableDictionary alloc] initWithCapacity:1];
	
	cachedUILanguage_=[[self nativeNameForLanguage:[[[NSBundle mainBundle] preferredLocalizations] objectAtIndex:0]] retain];
	
	cachedLicensesDictionary_=[[NSMutableDictionary alloc] initWithCapacity:1];
	
	if (bundle_!=nil)
	{
		NSString * tResourcesPath;
		NSUInteger tLanguageCount;
		NSMutableArray * tLanguagesArray;
		
		[IBtextView_ setTextContainerInset:NSMakeSize(6.0f,6.0f)];
	
		[IBtextView_ setDrawsBackground:NO];
		
		[[IBtextView_ enclosingScrollView] setDrawsBackground:NO];
		
		tResourcesPath=[bundle_ resourcePath];
		
		tLanguagesArray=[NSMutableArray array];
		
		if (tResourcesPath!=nil)
		{
			NSFileManager * tFileManager;
			NSArray * tArray;
			NSUInteger i,tCount;
			
			tFileManager=[NSFileManager defaultManager];
			
			tArray=[tFileManager directoryContentsAtPath:tResourcesPath];
			
			tCount=[tArray count];
			
			conversionDictionary_=[[NSMutableDictionary alloc] initWithCapacity:10];
			
			for(i=0;i<tCount;i++)
			{
				NSString * tFileName;
				BOOL isDirectory;
				NSString * tLprojFolderPath;
				
				tFileName=[tArray objectAtIndex:i];
				
				tLprojFolderPath=[tResourcesPath stringByAppendingPathComponent:tFileName];
				
				if ([tFileManager fileExistsAtPath:tLprojFolderPath isDirectory:&isDirectory]==YES && isDirectory==YES)
				{
					if ([tFileName hasPrefix:@"."]==NO && [[tFileName pathExtension] isEqualToString:@"lproj"]==YES)
					{
						NSString * tFolderLanguage;
						NSArray * tLanguageFolderContents;
						NSUInteger j,tLocalizedCount;
						NSDictionary * tLocalizedStringDictionary;
						NSString * tLocalizedStringPath;
						NSString * tLanguageKey;
						
						tFolderLanguage=[[tFileName lastPathComponent] stringByDeletingPathExtension];
						
						// Get the Localizations for this language
						
						tLocalizedStringPath=[tLprojFolderPath stringByAppendingPathComponent:@"Localizable.strings"];
						
						if ([tFileManager fileExistsAtPath:tLocalizedStringPath isDirectory:&isDirectory]==YES && isDirectory==NO)
						{
							tLocalizedStringDictionary=[NSDictionary dictionaryWithContentsOfFile:tLocalizedStringPath];
							
							if (tLocalizedStringDictionary!=nil)
							{
								[cachedLocalizedStrings_ setObject:tLocalizedStringDictionary forKey:tFolderLanguage];
							}
						}
						
						tLanguageKey=[self nativeNameForLanguage:tFolderLanguage];
						
						if (tLanguageKey!=nil)
						{
							[conversionDictionary_ setObject:tFolderLanguage forKey:tLanguageKey];
						}
						
						// Look for a license file
						
						tLanguageFolderContents=[tFileManager directoryContentsAtPath:tLprojFolderPath];
						
						tLocalizedCount=[tLanguageFolderContents count];
						
						for(j=0;j<tLocalizedCount;j++)
						{
							NSString * tLocalizedResourceName;
							
							tLocalizedResourceName=[tLanguageFolderContents objectAtIndex:j];
							
							if ([tFileManager fileExistsAtPath:[tLprojFolderPath stringByAppendingPathComponent:tLocalizedResourceName] isDirectory:&isDirectory]==YES)
							{
								if (isDirectory==YES)
								{
									// We only support RTFD
									
									if ([tLocalizedResourceName isEqualToString:@"License.rtfd"]==YES)
									{
										[tLanguagesArray addObject:tFolderLanguage];
										
										[cachedLicensesDictionary_ setObject:[tLprojFolderPath stringByAppendingPathComponent:tLocalizedResourceName] forKey:tFolderLanguage];
									}
								}
								else if (isDirectory==NO)
								{
									// We support RTF and TXT
									
									if ([tLocalizedResourceName isEqualToString:@"License.rtf"]==YES ||
										[tLocalizedResourceName isEqualToString:@"License.txt"]==YES)
									{
										[tLanguagesArray addObject:tFolderLanguage];
										
										[cachedLicensesDictionary_ setObject:[tLprojFolderPath stringByAppendingPathComponent:tLocalizedResourceName] forKey:tFolderLanguage];
									}
								}
							}
						}
					}
				}
			}
		}
		
		tLanguageCount=[cachedLicensesDictionary_ count];
		
		if (tLanguageCount>0)
		{
			sortedAvailableLanguages_=[[NSMutableArray alloc] initWithCapacity:tLanguageCount];
			
			if (sortedAvailableLanguages_!=nil)
			{
				NSUInteger i;
				
				for(i=0;i<tLanguageCount;i++)
				{
					NSString * tLanguageObject;
					NSString * tLanguageKey;
					
					tLanguageObject=[tLanguagesArray objectAtIndex:i];
					
					tLanguageKey=[self nativeNameForLanguage:tLanguageObject];
					
					if (tLanguageKey!=nil)
					{
						[sortedAvailableLanguages_ addObject:tLanguageKey];
					}
				}
			}
			
			// Build the Popup menu
		
			if ([sortedAvailableLanguages_ count]>0)
			{
				[sortedAvailableLanguages_ sortUsingSelector:@selector(caseInsensitiveCompare:)];
			
				[IBlanguagePopupButton_ removeAllItems];
				
				[IBlanguagePopupButton_ addItemsWithTitles:sortedAvailableLanguages_];
				
				cachedLicenseLanguage_=[[self nativeNameForLanguage:[[NSBundle preferredLocalizationsFromArray:tLanguagesArray] objectAtIndex:0]] retain];
			
				if (cachedLicenseLanguage_!=nil)
				{
					[IBlanguagePopupButton_ selectItemWithTitle:cachedLicenseLanguage_];
				}
			
				[self showLicenseForNativeLanguage:cachedLicenseLanguage_];
			}
		}
		else
		{
			[IBtextView_ setString:@"Missing License documents"];
		}
	}
	else
	{
		NSLog(@"Unable to create the bundle instance for the plugin");
	}
}

- (NSString *) nativeLocalizedString:(NSString *) inString forLanguage:(NSString *) inLanguage
{
	if (inLanguage!=nil && inString!=nil)
	{
		NSString * tLocalizedString;
		
		tLocalizedString=[[cachedLocalizedStrings_ objectForKey:[conversionDictionary_ objectForKey:inLanguage]] objectForKey:inString];
		
		if (tLocalizedString!=nil)
		{
			return tLocalizedString;
		}
	}
	
	return inString;
}

- (NSString *) nativeNameForLanguage:(NSString *) inLanguage
{
	if (inLanguage!=nil)
	{
		static NSDictionary * conversionDictionary=nil;
	
		if (conversionDictionary==nil)
		{
			NSString * tPath;
			
			tPath=[bundle_ pathForResource:@"ConversionTable" ofType:@"plist"];
		
			if (tPath!=nil)
			{
				conversionDictionary=[[NSDictionary alloc] initWithContentsOfFile:tPath];
			}
			else
			{
				NSLog(@"Missing ConversionTable.plist file");
			}
		}
		
		return [conversionDictionary objectForKey:inLanguage];
	}
	
	return nil;
}

- (NSString *) title
{
	return [[NSBundle bundleForClass:[self class]] localizedStringForKey:@"PaneTitle" value:nil table:nil];
}

- (id) bottomContentView
{
	return IBbottomView_;
}

#pragma mark -

- (void) didExitPane:(InstallerSectionDirection) inDirection
{
	[[NSNotificationCenter defaultCenter] removeObserver:self  name:NSViewFrameDidChangeNotification object:[window_ contentView]];
}

- (void) didEnterPane:(InstallerSectionDirection) inDirection
{
	window_=[IBlanguagePopupButton_ window];
	
	[self viewFrameDidChange:nil];
	
	[self updateButtonsForNativeLanguage:cachedLicenseLanguage_];
	
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(viewFrameDidChange:) 
												 name:NSViewFrameDidChangeNotification 
											  object:[window_ contentView]];
}

- (BOOL) shouldExitPane:(InstallerSectionDirection) inDirection
{
	if (InstallerDirectionForward==inDirection)
	{
		if (licenseAgreed_==NO)
		{
			NSString * tLanguage=cachedLicenseLanguage_;
			
			if (cachedUILanguage_!=nil && [sortedAvailableLanguages_ indexOfObject:cachedUILanguage_]==NSNotFound && [sortedAvailableLanguages_ count]<2)
			{
				NSDictionary * tDictionary;
				
				tDictionary=[cachedLocalizedStrings_ objectForKey:[conversionDictionary_ objectForKey:cachedUILanguage_]];
				
				if (tDictionary!=nil)
				{
					tLanguage=cachedUILanguage_;
				}
			}
			
			[IBalertTitle_ setStringValue:[self nativeLocalizedString:@"Alert Title" forLanguage:tLanguage]];
			
			[IBalertMessage_ setStringValue:[self nativeLocalizedString:@"Alert Message" forLanguage:tLanguage]];
			
			[IBalertAgreeButton_ setTitle:[self nativeLocalizedString:@"Agree" forLanguage:tLanguage]];
			
			[IBalertDisagreeButton_ setTitle:[self nativeLocalizedString:@"Disagree" forLanguage:tLanguage]];
			
			[IBalertReadLicenseButton_  setTitle:[self nativeLocalizedString:@"Read License" forLanguage:tLanguage]];
			
			[NSApp beginSheet:IBalertWindow_
			   modalForWindow:window_
				modalDelegate:nil
			   didEndSelector:nil
				  contextInfo:NULL];
		
			return NO;
		}
	}
	
	return YES;
}

- (IBAction) endDialog:(id) sender
{
	[NSApp endSheet:IBalertWindow_];
	
	[IBalertWindow_ orderOut:self];
	
	if ([sender tag]==1)
	{
		licenseAgreed_=YES;
		
		[self gotoNextPane];
	}
	else if ([sender tag]==2)
	{
		[NSApp terminate:self];
	}
}

#pragma mark -

- (IBAction) print:(id) sender
{
	NSPrintInfo * tPrintInfo;
	
	tPrintInfo=[NSPrintInfo sharedPrintInfo];
	
	if (tPrintInfo!=nil)
	{
		[tPrintInfo setVerticallyCentered:NO];
		
		[tPrintInfo setLeftMargin:50.0f];

		[tPrintInfo setRightMargin:50.0f];

		[tPrintInfo setTopMargin:55.0f];

		[tPrintInfo setBottomMargin:55.0f];
	
		[tPrintInfo setHorizontalPagination:NSFitPagination];
		
		[tPrintInfo setVerticalPagination:NSAutoPagination];
	}
	
	[[NSPrintOperation printOperationWithView:IBtextView_] runOperationModalForWindow:window_ 
																			 delegate:nil
																	   didRunSelector:nil
																	      contextInfo:NULL];
}

- (IBAction) saveAs:(id) sender
{
	NSSavePanel * tSavePanel;
	NSInteger tReturnCode;
	
	tSavePanel=[NSSavePanel savePanel];
	
	if (tSavePanel!=nil)
	{
		[tSavePanel setRequiredFileType:@"pdf"];

		tReturnCode=[tSavePanel runModalForDirectory:NSHomeDirectory() file:[self nativeLocalizedString:@"License" forLanguage:cachedLicenseLanguage_]];
		
		if (tReturnCode==NSFileHandlingPanelOKButton)
		{
			NSString * tPath;
			NSPrintInfo * tPrintInfo;
			BOOL tFileSaved=NO;
			
			tPath=[tSavePanel filename];
	
			tPrintInfo=[NSPrintInfo sharedPrintInfo];
			
			if (tPrintInfo!=nil)
			{
				NSMutableDictionary * tMutableDictionary;
				NSTextView * tTextView;
				
				[tPrintInfo setHorizontallyCentered:YES];
				
				[tPrintInfo setVerticallyCentered:NO];
				
				[tPrintInfo setLeftMargin:50.0f];

				[tPrintInfo setRightMargin:50.0f];

				[tPrintInfo setTopMargin:55.0f];

				[tPrintInfo setBottomMargin:55.0f];
			
				[tPrintInfo setHorizontalPagination:NSFitPagination];
				
				[tPrintInfo setVerticalPagination:NSAutoPagination];
				
				[tPrintInfo setJobDisposition:NSPrintSaveJob];
				
				
				tMutableDictionary= [tPrintInfo dictionary];
				
				if (tMutableDictionary!=nil)
				{
					[tMutableDictionary setObject:tPath forKey:NSPrintSavePath];
					
					[tMutableDictionary setObject:[NSNumber numberWithBool:YES ] forKey:NSPrintAllPages]; 
				}
				
				// We use a temporary text view (instead of the one in the pane because we need to set the text view to be as high as needed)
				
				tTextView=[[NSTextView alloc] initWithFrame:NSMakeRect(0.0f,0.0f,630.0f,1000.0f)];		// 630.0f looks fine
				
				if (tTextView!=nil)
				{
					NSTextContainer * tTextContainer;
					NSLayoutManager * tLayoutManager;
					NSUInteger tNumberOfGlyphs;
					NSRect tBounds;
					NSPrintOperation * tPrintOperation;
					
					tTextContainer=[IBtextView_ textContainer];
					
					// We will switch between the real text view and this one
					
					[tTextContainer setTextView:tTextView];
					
					tLayoutManager=[tTextContainer layoutManager];
					
					tNumberOfGlyphs=[tLayoutManager numberOfGlyphs];
					
					tBounds=[tLayoutManager boundingRectForGlyphRange:NSMakeRange(0,tNumberOfGlyphs) inTextContainer:tTextContainer];
				
					[tTextView setFrame:tBounds];
				
					tPrintOperation=[NSPrintOperation printOperationWithView:tTextView printInfo:tPrintInfo];
				
					if (tPrintOperation!=nil)
					{
						[tPrintOperation setShowsPrintPanel:NO];
			
						[tPrintOperation setShowsProgressPanel:NO];
						
						if ([tPrintOperation runOperation]==YES)
						{
							tFileSaved=YES;
						}
					}
					
					// We revert to the real view
					
					[tTextContainer setTextView:IBtextView_];
					
					[tTextView release];
				}
			}
			
			if (tFileSaved==NO)
			{
				NSBeep();
				
				NSBeginAlertSheet(NSLocalizedStringFromTableInBundle(@"Save Error Title",@"Localizable",bundle_,@""),
								  nil,
								  nil,
								  nil,
								  window_,
								  nil,
								  nil,
								  nil,
								  NULL,
								  NSLocalizedStringFromTableInBundle(@"Save Error Message",@"Localizable",bundle_,@""));
			}
		}
	}
	
	
}

- (IBAction) switchLicenseLanguage:(id) sender
{
	NSString * tSelectedLanguage;
	
	tSelectedLanguage=[sender titleOfSelectedItem];
	
	if (cachedLicenseLanguage_==nil || [cachedLicenseLanguage_ isEqualToString:tSelectedLanguage]==NO)
	{
		[cachedLicenseLanguage_ release];
		
		cachedLicenseLanguage_=[tSelectedLanguage retain];
		
		[cachedUILanguage_ release];
		
		cachedUILanguage_=[cachedLicenseLanguage_ retain];
		
		[self showLicenseForNativeLanguage:cachedLicenseLanguage_];
	}
}

- (void) showLicenseForNativeLanguage:(NSString *) inLanguage
{
	NSString * tDocumentPath;
		
	tDocumentPath=[cachedLicensesDictionary_ objectForKey:[conversionDictionary_ objectForKey:inLanguage]];
	
	// Update Text View
	
	if (tDocumentPath!=nil)
	{
		NSTextStorage * tTextStorage;
		BOOL success;
		
		[IBtextView_ setString:@""];
		
		tTextStorage=[IBtextView_ textStorage];
		
		[tTextStorage beginEditing];
		
		success = [tTextStorage readFromURL:[NSURL fileURLWithPath:tDocumentPath]
									options:[EULA_PluginPane defaultStorageOptionsWithPath:tDocumentPath]
						 documentAttributes:NULL];
		
		
		[tTextStorage endEditing];
		
		[IBtextView_ scrollRangeToVisible:NSMakeRange(0,0)];
	}
	
	[self updateButtonsForNativeLanguage:inLanguage];
}

- (void) updateButtonsForNativeLanguage:(NSString *) inNativeLanguage
{
	NSString * tLanguage=cachedLicenseLanguage_;
	
	if (cachedUILanguage_!=nil && [sortedAvailableLanguages_ indexOfObject:cachedUILanguage_]==NSNotFound)
	{
		NSDictionary * tDictionary;
		
		tDictionary=[cachedLocalizedStrings_ objectForKey:[conversionDictionary_ objectForKey:cachedUILanguage_]];
		
		if (tDictionary!=nil)
		{
			tLanguage=cachedUILanguage_;
		}
	}
	
	[self setTitle:[self nativeLocalizedString:@"PaneTitle" forLanguage:tLanguage]];

	// Update Buttons
	
	[IBprintButton_ setTitle:[self nativeLocalizedString:@"Print..." forLanguage:tLanguage]];
	
	[IBsaveButton_ setTitle:[self nativeLocalizedString:@"Save..." forLanguage:tLanguage]];
	
	
	[self setPreviousTitle:[self nativeLocalizedString:@"Go Back" forLanguage:tLanguage]];
	
	[self setNextTitle:[self nativeLocalizedString:@"Continue" forLanguage:tLanguage]];
}

#pragma mark -

- (void) viewFrameDidChange:(NSNotification *) inNotification
{
	// We want to make sure the no display text label is centered
	
	NSRect tViewFrame;
	NSRect tLabelFrame;
	
	// Center the Language Popup button
	
	tViewFrame=[[IBlanguagePopupButton_ superview] frame];
	
	tLabelFrame=[IBlanguagePopupButton_ frame];
	
	tLabelFrame.origin.x=_CGFloatRound(NSMidX(tViewFrame)-NSWidth(tLabelFrame)*0.5f)-2.f;
	
	[IBlanguagePopupButton_ setFrame:tLabelFrame];
}

@end
